#!/usr/bin/env python2.5
# -*- coding: latin-1 -*-

"""
GPTVview by asinus <andersejdeling@yahoo.se>

Version 1.0 (2007-11-11)

"""

import gtk 
import gobject
import string, urllib
from xml.dom.minidom import parse
import os

try:
    import hildon
except ImportError:
    is_maemo = False
else:
    is_maemo = True

GP_URL = 'http://gptv.abcdn.net.n2.abcdn.net/Senast_publicerat/.xml'
# GP_URL = 'senaste.xml'

def getText(nodelist):
    rc = ""
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data
    return rc

class GPvidlist:

    def __init__(self, url):
        f = urllib.urlopen( url )
        self.dom = parse(f)
        f.close()
        self.files = self.dom.getElementsByTagName("file")
        
    def getTitles(self):
        r = []
        for f in self.files:
            n = f.getElementsByTagName("title")[0]
            d = f.getElementsByTagName("published")[0]
            r.append( (getText(n.childNodes), getText(d.childNodes)) )
        return r
        
    def getURL(self, pos):
        n = self.files[pos].getElementsByTagName("URL")[0]
        plsfile = urllib.urlopen( getText(n.childNodes) )
        pls = parse(plsfile)
        clips = pls.getElementsByTagName("clip")
        urls = ""
        for c in clips:
            if c.getAttribute("mode") == "regular":
                n = c.getElementsByTagName("URL")[0]
                urls += getText(n.childNodes) + " "
        return urls
                
    def getPicDesc(self, pos):
        desc = self.files[pos].getElementsByTagName("desc")[0]
        th = self.files[pos].getElementsByTagName("thumbnails")[0]
        url = th.getElementsByTagName("item")[0]
        return ( getText(desc.childNodes), "http://" + getText(url.childNodes) )

class GPTVview:
    def __init__(self):
        self.selected_index = -1
    
        if is_maemo:
            self.window = hildon.Window()
            gtk.set_application_name("GPTVview")
            self.window.connect("window-state-event", self.on_window_state_change)
        else:
            self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
            self.window.set_default_size(800,480)
            self.window.set_title("GPTVview")  

        self.window.connect("destroy", gtk.main_quit)
        self.window.connect("key-press-event", self.on_key_press)
        self.window_in_fullscreen = False #The window isn't in full screen mode initially
    
        vbox = gtk.VBox(False, 5)
        self.window.add( vbox )
         
        frame = gtk.Frame("Status")
        self.label_stat = gtk.Label()
        frame.add(self.label_stat)
        vbox.pack_start(frame, False, True, 0)

        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        vbox.pack_start(sw, True, True, 0)

        self.listmodel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_INT)
        listbox = gtk.TreeView( self.listmodel )
        column = gtk.TreeViewColumn("Senaste filmerna", gtk.CellRendererText(), text=0)
        listbox.append_column(column)
        column = gtk.TreeViewColumn("Publicerad", gtk.CellRendererText(), text=1)
        listbox.append_column(column)
        selection = listbox.get_selection()
        selection.connect("changed", self.on_listbox_changed)
        sw.add(listbox)

        hbox = gtk.HBox(False, 5)
        vbox.pack_start(hbox, False, False, 0)
        
        self.image = gtk.Image()
        self.image.set_size_request(171, 128)
        hbox.pack_start(self.image, False, False, 0)
        
        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        hbox.pack_start(sw, True, True, 0)

        self.tb_desc = gtk.TextBuffer()
        tv = gtk.TextView( self.tb_desc )
        tv.set_wrap_mode(gtk.WRAP_WORD)
        tv.set_editable(False)
        tv.set_property('can-focus', False)
        sw.add( tv )

        self.button_go = gtk.Button("Visa filmen")
        self.button_go.connect("clicked", self.on_button_go_clicked)
        self.button_go.set_size_request(-1, 50)
        vbox.pack_end(self.button_go, False, True, 0)

        self.window.show_all()
        
    def set_status(self, text):
        self.label_stat.set_text( text )
        while gtk.events_pending():
            gtk.main_iteration()
            
    def get_list(self):
        self.set_status( "Hämtar listan..." )
        try:
            self.gp = GPvidlist( GP_URL )
            i = 0
            for (t,d) in self.gp.getTitles():
                self.listmodel.append( (t, d, i) )
                i += 1
            self.listmodel.set_sort_column_id(1, gtk.SORT_DESCENDING)
            self.set_status( "Välj en film:" )
        except Exception,e:
            self.set_status( str(e) )

    def on_window_state_change(self, widget, event, *args):
        if event.new_window_state & gtk.gdk.WINDOW_STATE_FULLSCREEN:
            self.window_in_fullscreen = True
        else:
            self.window_in_fullscreen = False

    def on_key_press(self, widget, event, *args):
        if event.keyval == gtk.keysyms.F6:
            # The "Full screen" hardware key has been pressed
            if self.window_in_fullscreen:
                self.window.unfullscreen ()
            else:
                self.window.fullscreen ()
        elif event.keyval == gtk.keysyms.Return:
            self.on_button_go_clicked(None)

    def on_listbox_changed(self, selection):
        # Get the selection iter
        model, selection_iter = selection.get_selected()
        if (selection_iter):
            try:
                self.selected_index = self.listmodel.get_value(selection_iter, 2)
                (desc, url) = self.gp.getPicDesc(self.selected_index)
                self.tb_desc.set_text( desc )

                f=urllib.urlopen( url )
                pbl = gtk.gdk.PixbufLoader()
                pbl.set_size(171, 128)
                pbl.write(f.read())
                pbl.close()
                self.image.set_from_pixbuf( pbl.get_pixbuf() )
            except Exception,e:
                self.set_status( str(e) )

        else:
            self.selected_index = -1
    
    def on_button_go_clicked(self, widget):
        if -1 != self.selected_index:
            self.set_status( "Startar uppspelning..." )
            # print self.gp.getURL(index)
            command = 'mplayer -really-quiet -framedrop -cache 2048 ' + self.gp.getURL(self.selected_index)
            os.system( command )
            self.set_status( "Välj en ny film:" )
            
    def run(self):
        self.get_list()
        gtk.main() 
                                        
if __name__ == "__main__":  
    app = GPTVview() 
    app.run()

